-- ArchimedesTechBoostTrait
-- Author: gkarjr Steam / PonderThis civfanatics
-- DateCreated: 5/5/2017 8:36:45 AM
-- Description: This implements Archimedes special trait.
--   At the beginning of every turn, if (1) there is currently a tech being researched,
--   (2) the boost for that tech has NOT been triggered, and
--   (3) the random generator rolls a 20 (from 1 to 20)
--   THEN player gets a tech boost.
--------------------------------------------------------------

function GetTechProgress(playerConfig:table, localPlayer)
	if (localPlayer == nil) then
		return -1, false;	-- currentTechID, techIsBoosted
	end

	print("GetTechProgress for civilization " .. playerConfig:GetCivilizationTypeName() .. 
		" leader " .. playerConfig:GetLeaderTypeName());

	local techIsBoosted;
	local playerTechs = localPlayer:GetTechs();
	local currentTechID :number = playerTechs:GetResearchingTech();

	if currentTechID >= 0 then
		-- techProgress = playerTechs:GetResearchProgress(currentTechID); -- Turns out I don't need this
		-- print("  Current Progress is " .. techProgress);
		-- techCanBoost = playerTechs:CanTriggerBoost(currentTechID);	-- Nope, function expected instead of nil means you can't have it. That's OK I don't need it anyway.
		techIsBoosted = playerTechs:HasBoostBeenTriggered(currentTechID);
	else
		-- techProgress = 0;
		techIsBoosted = false;
	end

	print("-- Returning currentTechID of " .. currentTechID .. 
		", techIsBoosted of " .. tostring(techIsBoosted));

	return currentTechID, techIsBoosted;
end

function MaybeBoostTech(localPlayer)
	-- Coming into here we should know that currentTechID >= 0 and therefore localPlayer is not nil.
	-- So now do the random check (5%, or 1 in 20)
	if (math.random(1, 20) == 20) then
		local playerTechs = localPlayer:GetTechs();
		local currentTechID: number = playerTechs:GetResearchingTech();

		print("MaybeBoostTech is BOOSTING tech with ID " .. currentTechID);
		playerTechs:TriggerBoost(currentTechID);
	else
		print("MaybeBoostTech random number says no boost for you.");
	end
end

function StartOfTurn(...)
	local args = {...};
	local playerID = args[1];
	local currentTurn = Game.GetCurrentGameTurn();
	local playerConfig:table = PlayerConfigurations[playerID];
	local playerLeaderName:string = playerConfig:GetLeaderTypeName();
	local localPlayer = Players[playerID];
	local localPlayerStatus: string;

	if (playerLeaderName == nil) then
		playerLeaderName = "?";
	end

	if (localPlayer == nil) then
		localPlayerStatus = ", Players[playerID] is nil";
	else
		localPlayerStatus = ", Players[playerID] exists";
	end

	if playerLeaderName ~= 'LEADER_GJPT_ARCHIMEDES' then
	    -- If this player is *not* archimedes, none of this applies!
		return;
	end

	print("StartOfTurn for Turn " .. currentTurn .. ", playerID " .. playerID .. ", Leader " .. playerLeaderName
		.. localPlayerStatus);

	local currentTechID, techIsBoosted = GetTechProgress(playerConfig, localPlayer);
	-- FYI, if localPlayer is nil, GetTechProgress() returns -1 for currentTechID.
	if (currentTechID >= 0) and not techIsBoosted then
		print("-- Player's tech *is* boost eligible.");
		MaybeBoostTech(localPlayer);
	else
		print("-- NOT calling MaybeBoostTech().");
	end
end

-- And the code that hooks these functions up:
print("Hooking up StartOfTurn to Events.PlayerTurnActivated");
Events.PlayerTurnActivated.Add( StartOfTurn );
print("Event hooked up");